@@ -8,13 +8,13 @@ super_replacer:
     db_name: lua/replacer
     delimiter: "|"
     comment_format: "〔%s〕"
-    chain: true   #true表示流水线作业，上一个option产出交给下一个处理，典型的s2t>t2hk=s2hk，false就是并行，直接用text转换
+    chain: true  #true表示流水线作业，上一个option产出交给下一个处理，典型的s2t>t2hk=s2hk，false就是并行，直接用text转换
     types:
       # 场景1：输入 '哈哈' -> 变成 '1.哈哈 2.😄'
-      - option: emoji           # 开关名称与上面开关名称保持一致
+      - option: emoji          # 开关名称与上面开关名称保持一致
         mode: append            # 新增候选append 替换原候选replace 替换注释comment
         comment_mode: none      # 注释模式: "append"(原候选注释继承), "text"(原候选文本放在注释), "none"(空，默认)
-        tags: [abc]             # 生效的tag
+        tags: [abc]            # 生效的tag
         prefix: "_em_"          # 前缀用于区分同一个数据库的不同用途数据
         files:
           - lua/data/emoji.txt
@@ -29,15 +29,15 @@ super_replacer:
           - lua/data/chinese_english.txt
       # 场景3：用于常驻的直接替换 option: true
       - option: true
-        mode: append         # <--- 新增候选模式
+        mode: append        # <--- 新增候选模式
         comment_mode: none
         tags: [abc]
         prefix: "_ot_"
         files:
           - lua/data/others.txt
       # 场景4：用于简繁转换的直接替换
-      - option: [ s2t, s2hk, s2tw ]   #后面依赖这条流水线有一个开关为true这条流水线就能工作
-        mode: replace         # <--- 替换原候选模式
+      - option: [ s2t, s2hk, s2tw ]  #后面依赖这条流水线有一个开关为true这条流水线就能工作
+        mode: replace        # <--- 替换原候选模式
         comment_mode: append
         sentence: true        # <--- 句子级别替换
         tags: [abc]
@@ -46,7 +46,7 @@ super_replacer:
           - lua/data/STCharacters.txt
           - lua/data/STPhrases.txt
       - option: s2hk
-        mode: replace         # <--- 替换原候选模式
+        mode: replace        # <--- 替换原候选模式
         comment_mode: append
         sentence: true        # <--- 句子级别替换
         tags: [abc]
@@ -55,7 +55,7 @@ super_replacer:
           - lua/data/HKVariants.txt
           - lua/data/HKVariantsRevPhrases.txt
       - option: s2tw
-        mode: replace         # <--- 替换原候选模式
+        mode: replace        # <--- 替换原候选模式
         comment_mode: append
         sentence: true        # <--- 句子级别替换
         tags: [abc]
@@ -63,7 +63,7 @@ super_replacer:
         files:
           - lua/data/TWVariants.txt
           - lua/data/TWVariantsRevPhrases.txt
-      - option: [ abbrev_lazy, abbrev_always ] 
+      - option: [ abbrev_lazy, abbrev_always ]
         mode: abbrev          # <--- 新增的简码模式
         tags: [abc]
         prefix: "_abbr_"
@@ -71,6 +71,7 @@ super_replacer:
           - lua/data/abbrev.txt # 格式：zm\t怎么|在吗
 ]]
 
+
 local M = {}
 
 -- 性能优化：本地化常用库函数
@@ -86,6 +87,7 @@ local s_upper = string.upper
 local open = io.open
 local type = type
 local tonumber = tonumber
+local replacer_instance = nil
 
 -- 基础依赖
 local function safe_require(name)
@@ -97,6 +99,23 @@ end
 local userdb = safe_require("lib/userdb") or safe_require("userdb")
 local wanxiang = safe_require("wanxiang")
 
+-- UTF-8 辅助
+local function get_utf8_offsets(text)
+    local offsets = {}
+    local len = #text
+    local i = 1
+    while i <= len do
+        insert(offsets, i)
+        local b = s_byte(text, i)
+        if b < 128 then i = i + 1
+        elseif b < 224 then i = i + 2
+        elseif b < 240 then i = i + 3
+        else i = i + 4 end
+    end
+    insert(offsets, len + 1)
+    return offsets
+end
+
 -- 重建数据库 (仅在 wanxiang 版本变更时运行)
 local function rebuild(tasks, db)
     if db.empty then db:empty() end
@@ -115,28 +134,59 @@ local function rebuild(tasks, db)
                 end
             end
             f:close()
-        else
-            if log and log.info then log.info("super_replacer: 无法读取文件: " .. txt_path) end
         end
     end
     return true
 end
 
--- UTF-8 辅助
-local function get_utf8_offsets(text)
-    local offsets = {}
-    local len = #text
-    local i = 1
-    while i <= len do
-        insert(offsets, i)
-        local b = s_byte(text, i)
-        if b < 128 then i = i + 1
-        elseif b < 224 then i = i + 2
-        elseif b < 240 then i = i + 3
-        else i = i + 4 end
+-- 连接或重连数据库 (Singleton Logic)
+local function connect_db(db_name, current_version, delimiter, tasks)
+    -- 1. 检查现有实例是否存活
+    if replacer_instance then
+        local status, _ = pcall(function() return replacer_instance:fetch("___test___") end)
+        if status then return replacer_instance end
+        replacer_instance = nil -- 死链接，重置
     end
-    insert(offsets, len + 1)
-    return offsets
+
+    if not userdb then return nil end
+    
+    -- 2. 创建 LevelDB 对象
+    local db = userdb.LevelDb(db_name)
+    if not db then return nil end
+
+    -- 3. 检查是否需要重建 (先尝试只读打开)
+    local needs_rebuild = false
+    if db:open_read_only() then
+        local db_ver = db:meta_fetch("_wanxiang_ver") or ""
+        local db_delim = db:meta_fetch("_delim")
+        if db_ver ~= current_version or db_delim ~= delimiter then
+            needs_rebuild = true
+        end
+        db:close() -- 必须关闭才能切换模式
+    else
+        needs_rebuild = true -- 无法打开，强制重建
+    end
+
+    -- 4. 执行重建 (Write Mode)
+    if needs_rebuild then
+        if db:open() then
+            rebuild(tasks, db)
+            db:meta_update("_wanxiang_ver", current_version)
+            db:meta_update("_delim", delimiter)
+            if log and log.info then
+                log.info("super_replacer: 数据已更新至 " .. current_version)
+            end
+            db:close()
+        end
+    end
+
+    -- 5. 最终挂载 (Read-Only Mode)
+    if db:open_read_only() then
+        replacer_instance = db
+        return db
+    end
+    
+    return nil
 end
 
 -- FMM 分词转换算法
@@ -144,29 +194,29 @@ local function segment_convert(text, db, prefix, split_pat)
     local offsets = get_utf8_offsets(text)
     local char_count = #offsets - 1
     local result_parts = {}
-    local i = 1 
-    local MAX_LOOKAHEAD = 6 
-    
+    local i = 1
+    local MAX_LOOKAHEAD = 6
+  
     while i <= char_count do
         local matched = false
         local max_j = i + MAX_LOOKAHEAD
         if max_j > char_count + 1 then max_j = char_count + 1 end
-        
+      
         for j = max_j - 1, i + 1, -1 do
             local start_byte = offsets[i]
             local end_byte = offsets[j] - 1
             local sub_text = s_sub(text, start_byte, end_byte)
-            
+          
             local val = db:fetch(prefix .. sub_text)
             if val then
-                local first_val = s_match(val, split_pat) 
+                local first_val = s_match(val, split_pat)
                 insert(result_parts, first_val or sub_text)
-                i = j - 1 
+                i = j - 1
                 matched = true
                 break
             end
         end
-        
+      
         if not matched then
             local start_byte = offsets[i]
             local end_byte = offsets[i+1] - 1
@@ -183,14 +233,13 @@ local function segment_convert(text, db, prefix, split_pat)
     end
     return concat(result_parts)
 end
-
 -- 模块接口
 
 function M.init(env)
     local ns = env.name_space
     ns = s_gsub(ns, "^%*", "")
     local config = env.engine.schema.config
-    
+  
     local user_dir = rime_api:get_user_data_dir()
     local shared_dir = rime_api:get_shared_data_dir()
 
@@ -199,22 +248,22 @@ function M.init(env)
     local delim = config:get_string(ns .. "/delimiter") or "|"
     env.delimiter = delim
     env.comment_format = config:get_string(ns .. "/comment_format") or "〔%s〕"
-    
+  
     -- 获取全局版本号
     local current_version = "v0.0.0"
     if wanxiang and wanxiang.version then
         current_version = wanxiang.version
     end
-    
+  
     env.chain = config:get_bool(ns .. "/chain")
     if env.chain == nil then env.chain = false end
 
     if delim == " " then env.split_pattern = "%S+"
     else local esc = s_gsub(delim, "[%-%.%+%[%]%(%)%$%^%%%?%*]", "%%%1"); env.split_pattern = "([^" .. esc .. "]+)" end
 
-    -- 2. 解析 Types
+    -- 2. 解析 Types (这部分必须保留在 init，因为不同 schema 可能配置不同)
     env.types = {}
-    local tasks = {} -- 仅在需要重建时使用
+    local tasks = {} -- 用于重建数据库的文件列表
 
     local function resolve_path(relative)
         if not relative then return nil end
@@ -229,11 +278,11 @@ function M.init(env)
 
     local types_path = ns .. "/types"
     local type_list = config:get_list(types_path)
-    
+  
     if type_list then
         for i = 0, type_list.size - 1 do
             local entry_path = types_path .. "/@" .. i
-            
+          
             -- 解析 triggers
             local triggers = {}
             local opts_keys = {"option", "options"}
@@ -246,16 +295,11 @@ function M.init(env)
                         if val then insert(triggers, val) end
                     end
                 else
-                    -- 1. 如果配置写的是 true (bool)，get_bool 返回 true，我们插入布尔值 true。
-                    -- 2. 如果配置写的是 s2t (string)，get_bool 返回 false (或nil)，我们进入 else 读字符串。
-                    if config:get_bool(key_path) == true then 
-                        insert(triggers, true) 
+                    if config:get_bool(key_path) == true then
+                        insert(triggers, true)
                     else
                         local val = config:get_string(key_path)
-                        -- 只有当它不是 "true" 字符串时才插入，防止双重解析（虽然上面的if已经拦截了）
-                        if val and val ~= "true" then
-                            insert(triggers, val)
-                        end
+                        if val and val ~= "true" then insert(triggers, val) end
                     end
                 end
             end
@@ -274,9 +318,9 @@ function M.init(env)
                     end
                 else
                     local val = config:get_string(key_path)
-                    if val then 
+                    if val then
                         if not target_tags then target_tags = {} end
-                        target_tags[val] = true 
+                        target_tags[val] = true
                     end
                 end
             end
@@ -293,12 +337,12 @@ function M.init(env)
                     triggers = triggers,
                     tags = target_tags,
                     prefix = prefix,
-                    mode   = mode,
+                    mode  = mode,
                     comment_mode = comment_mode,
                     fmm = fmm
                 })
 
-                -- 收集文件路径 (用于重建)
+                -- 收集文件路径 (仅用于可能发生的 rebuild)
                 local keys_to_check = {"files", "file"}
                 for _, key in ipairs(keys_to_check) do
                     local d_path = entry_path .. "/" .. key
@@ -317,38 +361,18 @@ function M.init(env)
         end
     end
 
-    -- 3. DB 初始化
-    if not userdb then return end
-    local ok, db = pcall(function() local d = userdb.LevelDb(db_name); d:open(); return d end)
-
-    if ok and db then
-        env.db = db
-        local db_version = db:meta_fetch("_wanxiang_ver") or ""
-        local old_delim = db:meta_fetch("_delim")
-        local need_rebuild = false
-        if current_version ~= db_version then need_rebuild = true end
-        if env.delimiter ~= old_delim then need_rebuild = true end
-        
-        if need_rebuild then
-            if rebuild(tasks, db) then
-                db:meta_update("_wanxiang_ver", current_version)
-                db:meta_update("_delim", env.delimiter)
-                if log and log.info then 
-                    log.info("super_replacer: 检测到版本变更 (" .. db_version .. " -> " .. current_version .. ")，数据已重建。") 
-                end
-            end
-        end
-    else
-        env.db = nil
-    end
+    -- 3. DB 初始化 (使用单例连接)
+    env.db = connect_db(db_name, current_version, env.delimiter, tasks)
 end
 
 function M.fini(env)
-    if env.db then env.db:close(); env.db = nil end
+    -- 我们只是断开当前 env 对全局 db 的引用
+    env.db = nil
 end
 
--- [Core Function] 核心逻辑
+-- [Core Function] 核心逻辑 (保持原有逻辑不变)
 function M.func(input, env)
+    -- 如果数据库未连接，直接透传
     if not env.types or #env.types == 0 or not env.db then
         for cand in input:iter() do yield(cand) end
         return
@@ -361,7 +385,7 @@ function M.func(input, env)
     local comment_fmt = env.comment_format
     local is_chain = env.chain
     local input_code = ctx.input
-
+    local HIGH_THRESHOLD = 99
     local input_type = "unknown"
     if wanxiang and wanxiang.get_input_method_type then
         input_type = wanxiang.get_input_method_type(env)
@@ -369,23 +393,24 @@ function M.func(input, env)
 
     local seg = ctx.composition:back()
     local current_seg_tags = seg and seg.tags or {}
+    
     -- [Helper] 通用处理函数
     local function process_rules(cand)
         local current_text = cand.text
-        local show_main = true 
-        local current_main_comment = cand.comment 
-        
-        local pending_candidates = {} 
+        local show_main = true
+        local current_main_comment = cand.comment
+      
+        local pending_candidates = {}
         local comments = {}
-        
+      
         for _, t in ipairs(types) do
-            if t.mode ~= "abbrev" then -- 跳过 abbrev 模式
+            if t.mode ~= "abbrev" then
                 local is_active = false
                 for _, trigger in ipairs(t.triggers) do
                     if trigger == true then is_active = true; break
                     elseif type(trigger) == "string" and ctx:get_option(trigger) then is_active = true; break end
                 end
-                
+              
                 local is_tag_match = true
                 if t.tags then
                     is_tag_match = false
@@ -393,17 +418,17 @@ function M.func(input, env)
                         if current_seg_tags[req_tag] then is_tag_match = true; break end
                     end
                 end
-                
+              
                 if is_active and is_tag_match then
                     local query_text = is_chain and current_text or cand.text
                     local key = t.prefix .. query_text
                     local val = db:fetch(key)
-                    
+                  
                     if not val and t.fmm then
                         local seg_result = segment_convert(query_text, db, t.prefix, split_pat)
                         if seg_result ~= query_text then val = seg_result end
                     end
-                    
+                  
                     if val then
                         local mode = t.mode
                         local rule_comment = ""
@@ -414,12 +439,12 @@ function M.func(input, env)
                             local parts = {}
                             for p in s_gmatch(val, split_pat) do insert(parts, p) end
                             insert(comments, concat(parts, " "))
-                            
+                          
                         elseif mode == "replace" then
                             if is_chain then
                                 local first = true
                                 for p in s_gmatch(val, split_pat) do
-                                    if first then 
+                                    if first then
                                         current_text = p
                                         if t.comment_mode == "none" then current_main_comment = ""
                                         elseif t.comment_mode == "text" then current_main_comment = cand.text end
@@ -430,18 +455,18 @@ function M.func(input, env)
                                 end
                             else
                                 show_main = false
-                                for p in s_gmatch(val, split_pat) do 
-                                    insert(pending_candidates, { text=p, comment=rule_comment }) 
+                                for p in s_gmatch(val, split_pat) do
+                                    insert(pending_candidates, { text=p, comment=rule_comment })
                                 end
                             end
                         elseif mode == "append" then
-                            for p in s_gmatch(val, split_pat) do 
-                                insert(pending_candidates, { text=p, comment=rule_comment }) 
+                            for p in s_gmatch(val, split_pat) do
+                                insert(pending_candidates, { text=p, comment=rule_comment })
                             end
                         end
                     end
                 end
-            end 
+            end
         end
 
         if #comments > 0 then
@@ -457,7 +482,7 @@ function M.func(input, env)
         if show_main then
             if is_chain and current_text ~= cand.text then
                 local nc = Candidate(cand.type or "kv", cand.start, cand._end, current_text, current_main_comment)
-                nc.preedit = cand.preedit 
+                nc.preedit = cand.preedit
                 nc.quality = cand.quality
                 yield(nc)
             else
@@ -469,15 +494,29 @@ function M.func(input, env)
             if not (show_main and item.text == current_text) then
                 local nc = Candidate("derived", cand.start, cand._end, item.text, item.comment)
                 nc.preedit = cand.preedit
-                nc.quality = cand.quality 
+                nc.quality = cand.quality
                 yield(nc)
             end
         end
     end
-    -- [Helper] 尝试触发简码
-    local function try_trigger_abbrev(is_empty_override)
+
+    -- 核心状态变量
+    local pending_cands = {}
+    local seen_texts = {} -- 去重表
+    local limit = 10
+    local has_phrase = false
+    local cand_count = 0
+    local abbrev_triggered = false 
+
+    -- [Helper 1] 规则处理封装
+    local function process_and_record(cand)
+        seen_texts[cand.text] = true
+        process_rules(cand)
+    end
+
+    -- [Helper 2] 融合后的简码逻辑
+    local function try_trigger_abbrev_logic(is_empty_override, target_quality)
         for _, t in ipairs(types) do
-            -- 只有当模式是 abbrev 且 当前不是全拼 时，才进入逻辑
             if t.mode == "abbrev" and input_type ~= "pinyin" then
                 local is_tag_match = true
                 if t.tags then
@@ -509,75 +548,104 @@ function M.func(input, env)
                     end
 
                     if should_trigger then
-                         -- ... (查库逻辑) ...
                         local key = t.prefix .. input_code
                         local val = db:fetch(key)
                         if not val and not s_match(input_code, "[A-Z]") then
                             val = db:fetch(t.prefix .. s_upper(input_code))
                         end
+                        
                         if val then
                             for p in s_gmatch(val, split_pat) do
-                                local abbrev_cand = Candidate("abbrev", 0, #input_code, p, "")
-                                abbrev_cand.quality = 90
-                                process_rules(abbrev_cand)
+                                if not seen_texts[p] then
+                                    local abbrev_cand = Candidate("abbrev", 0, #input_code, p, "")
+                                    abbrev_cand.quality = target_quality
+                                    process_and_record(abbrev_cand)
+                                end
                             end
                         end
                     end
                 end
             end
         end
+        abbrev_triggered = true
     end
-    -- [Main Loop] 主循环
-    local pending_user_cands = {}  -- 用户词队列 (user_table)
-    local pending_other_cands = {} -- 原始前N个候选队列
-    local limit = 10               -- 探测窗口
-    local has_phrase = false
-    local cand_count = 0
 
+    -- [主循环]
     for cand in input:iter() do
         cand_count = cand_count + 1
+        local q = cand.quality or 0
         
-        if cand_count <= limit then
-            if cand.type == "phrase" then has_phrase = true end
-            if cand.type == "user_table" then
-                insert(pending_user_cands, cand)
-            else
-                insert(pending_other_cands, cand)
+        if cand.type == "phrase" then has_phrase = true end
+        
+        -- 1. [权重跳水/插队检测]
+        if not abbrev_triggered and q < HIGH_THRESHOLD and #pending_cands > 0 then
+            local max_q = 0
+            local has_high_q = false
+            for _, pc in ipairs(pending_cands) do 
+                local pq = pc.quality or 0
+                if pq > max_q then max_q = pq end
+                if pq > HIGH_THRESHOLD then has_high_q = true end
             end
+            
+            if has_high_q then
+                for _, pc in ipairs(pending_cands) do process_and_record(pc) end
+                pending_cands = {}
+                if not has_phrase then
+                    try_trigger_abbrev_logic(true, max_q - 0.001)
+                end
+            end
+        end
+
+        -- 2. [基础缓存逻辑]
+        if cand_count <= limit then
+            table.insert(pending_cands, cand)
         else
             if cand_count == limit + 1 then
-
-                for _, uc in ipairs(pending_user_cands) do
-                    process_rules(uc)
-                end
-
-                if not has_phrase then 
-                    try_trigger_abbrev(true) 
+                local has_high_q = false
+                for _, pc in ipairs(pending_cands) do 
+                    if (pc.quality or 0) > HIGH_THRESHOLD then 
+                        has_high_q = true 
+                        break
+                    end
                 end
                 
-                for _, oc in ipairs(pending_other_cands) do
-                    process_rules(oc)
+                if not abbrev_triggered then
+                    try_trigger_abbrev_logic(not has_phrase, has_high_q and 99 or 9999) 
                 end
-
-                pending_user_cands = {}
-                pending_other_cands = {}
+                for _, pc in ipairs(pending_cands) do process_and_record(pc) end
+                pending_cands = nil
             end
-            process_rules(cand)
+            process_and_record(cand)
         end
     end
 
-    -- 如果整个输入流都不到 limit 个词，循环结束后的收尾逻辑
-    if #pending_user_cands > 0 or #pending_other_cands > 0 or (cand_count > 0 and cand_count <= limit) then
+    -- 3. [收尾阶段]
+    if pending_cands then
+        if not abbrev_triggered then
+            local max_q = 0
+            local has_high_q = false
+            for _, pc in ipairs(pending_cands) do 
+                local pq = pc.quality or 0
+                if pq > max_q then max_q = pq end
+                if pq > HIGH_THRESHOLD then has_high_q = true end
+            end
 
-        for _, uc in ipairs(pending_user_cands) do
-            process_rules(uc)
+            if has_high_q then
+                for _, pc in ipairs(pending_cands) do process_and_record(pc) end
+                if not has_phrase then
+                    try_trigger_abbrev_logic(true, max_q - 0.001)
+                end
+            else
+                if not has_phrase then
+                    try_trigger_abbrev_logic(true, 9999)
+                end
+                for _, pc in ipairs(pending_cands) do process_and_record(pc) end
+            end
+            pending_cands = nil
         end
 
-        if not has_phrase then
-            try_trigger_abbrev(true)
-        end
-        for _, oc in ipairs(pending_other_cands) do
-            process_rules(oc)
+        if pending_cands then
+            for _, pc in ipairs(pending_cands) do process_and_record(pc) end
         end
     end
 end
